unit FormMain;

interface

uses
  System.SysUtils, System.Types, System.UITypes, System.Classes, System.Variants,
  FMX.Types, FMX.Controls, FMX.Forms, FMX.Graphics, FMX.Dialogs, FMXTee.Engine, FMXTee.Series, FMXTee.Procs,
  FMXTee.Chart, System.DateUtils, System.Math;

type
  TfrmMain = class(TForm)
    chMain: TChart;
    aSeries: TFastLineSeries;
    procedure FormCreate(Sender: TObject);
  private
    { Private declarations }
    FNumOfEntries: integer;
    procedure RealTimeAdd(Series: TChartSeries);
    procedure DoScrollPoints(Series: TChartSeries);
    procedure AdjustView(Series: TChartSeries);
  public
    { Public declarations }
  end;

const
  NUM_OF_SEC_T0_VEIW = 60;

var
  frmMain: TfrmMain;

implementation

{$R *.fmx}

procedure TfrmMain.FormCreate(Sender: TObject);
begin
  FNumOfEntries := 0;

  // Prepare chart
  chMain.ClipPoints                 := false;
  chMain.Title.Visible              := false;
  chMain.Legend.Visible             := false;
  chMain.LeftAxis.Axis.Width        := 1;
  chMain.BottomAxis.Axis.Width      := 1;
  chMain.BottomAxis.RoundFirstLabel := false;
  chMain.View3D                     := False;
  chMain.BevelOuter                 := bvNone;
  chMain.BottomAxis.DateTimeFormat  := 'hh:MM:ss';
  aSeries.XValues.DateTime          := true;
  chMain.BottomAxis.Increment       := IncSecond(0, NUM_OF_SEC_T0_VEIW div 4);
  chMain.LeftAxis.Increment         := 20;
  chMain.LeftAxis.AxisValuesFormat  := '0%';
  chMain.Zoom.Allow                 := false;
  chMain.AllowPanning               := pmNone;
  chMain.BufferedDisplay            := True;
  aSeries.AutoRepaint               := true;
  aSeries.XValues.Order             := loNone;
  chMain.Axes.FastCalc              := True;
  aSeries.DrawAllPoints             := false;
  // For Windows NT, 2000 and XP only: Speed realtime painting with solid pens of width 1.
  //Series1.FastPen                   := True;

  // Initialize axis scales; left axis values [0,100] %; bottom axis values are 'the last NumOfSecToView'
  chMain.LeftAxis.SetMinMax(0, 100);
  chMain.BottomAxis.SetMinMax(IncSecond(now, - NUM_OF_SEC_T0_VEIW), now);

end;

procedure TfrmMain.RealTimeAdd(Series: TChartSeries);
var
  XValue,YValue : Double;
  CurrentIndex: integer;
begin
  if Series.Count = 0 then begin  // First random point
    YValue := Random(50);
    XValue := now;
  end else begin
    // Next point
    YValue := Series.YValues.Last + Random(10) - 4.5;
    YValue := Max(Min(YValue, 100), 0);
    XValue := now;
  end;

  // Add new point
  CurrentIndex := Series.AddXY(XValue,YValue);

  // Add a 'Mark' for every 50th sample entered
  inc(FNumOfEntries);
  if FNumOfEntries mod 50 = 0 then begin
    Series.Marks.Item[CurrentIndex - 1].Text.Text := 'A'; //Just show an 'A' for now
    Series.Marks.Item[CurrentIndex - 1].Visible := true;
  end;

end;

procedure TfrmMain.DoScrollPoints(Series: TChartSeries);
begin
  // Delete 'past' points.
  if Series.XValues.Count > 0 then begin
    while Series.XValues.First < IncSecond(now, -NUM_OF_SEC_T0_VEIW) do begin
      Series.Delete(0);
    end;
  end;

  // Adjust 'view'
  AdjustView(Series);
end;

procedure TfrmMain.AdjustView(Series: TChartSeries);
begin
  Series.GetHorizAxis.SetMinMax(IncSecond(now, -NUM_OF_SEC_T0_VEIW), now);
end;

end.
